#zkmBatchDownloader stand-alone program

from zkm import zkmFile
from zkm import zkmHtml
from zkm import zkmUrl
from zkm.zkmZope import getOptionsDictFromZope
from zkm.zkmTrace import showError, writeTracebackAsString

import urllib
import string
import time
import sys
import os

fileList = []
optionsDict = {}

def downloadError(fullUrl, data):
	#might be bad link, so just log and go on
	showError("Could not download file at URL " + fullUrl + "\n" + `data`)
	return

def	downloadFile(fullUrl, filePathName):
	try:
		urllib.urlretrieve(fullUrl, filePathName)
		print ".",
	except KeyboardInterrupt, data:
		urllib.urlcleanup()
		raise
	except IOError, data:
		downloadError(fullUrl, data)
	except:
		downloadError(fullUrl, None)
		raise

	urllib.urlcleanup()
	return

def folderHierarchyFromUrl(baseFolder, fullUrl):
	filePathName = zkmUrl.filePathNameFromFullUrl(baseFolder, fullUrl)
	filePath = os.path.dirname(filePathName)

	if not os.path.isdir(filePath):
		os.makedirs(filePath)

	return filePathName

def downloadBinaries(filePathName):
	errCount = 0
	while errCount < 5:
		try:
			#have gotten IOError 13 permission denied at this point, which shuts down the
			#batch downloader. Has to be timing issue, so we'll delay and try up to five times
			#before failing.
			htmlFilePathName = zkmFile.readFileIntoString(filePathName)
			break
		except IOError, data:
			if data[0] == 13:	#permission denied error
				errCount = errCount + 1
				time.sleep(1)

	if errCount == 5:
		print "Cannot process (permission denied) file: " + filePathName
		return

	s = zkmFile.readFileIntoString(htmlFilePathName)
	zkmUrlFromFile = zkmHtml.findZkmUrl(s)
	urlBase = zkmUrl.getUrlBaseFromFullUrl(zkmUrlFromFile)
	imageUrlsList = zkmHtml.findImageFileUrls(s, urlBase)
	pdfUrlsList = zkmHtml.findPdfFileUrls(s, urlBase)
	binariesList = imageUrlsList + pdfUrlsList

	binariesFolder = optionsDict["binariesFolder"]

	for fullUrl in binariesList:
		filePathName = folderHierarchyFromUrl(binariesFolder, fullUrl)
		downloadFile(fullUrl, filePathName)

	return

def processNextTempFile(tempFolder):
	global fileList

	if len(fileList) == 0:
		fileList = os.listdir(tempFolder)

	while len(fileList) > 0:
		fileName = fileList[-1]
		del fileList[-1]

		if fileName[:6] == "zkmBd_":
			filePathName = tempFolder + "\\" + fileName
			downloadBinaries(filePathName)
			os.remove(filePathName)
			time.sleep(1)
			break

	return

def serve(zopeHost, zopePort):
	global optionsDict
	try:
		optionsDict = getOptionsDictFromZope(zopeHost, zopePort)
		tempFolder = optionsDict["tempFolder"]

		print "zkmBatchDownloader is ready. Each period indicates a downloaded file."

		while 1:
			time.sleep(1)
			processNextTempFile(tempFolder)

	except Exception, data:
		showError("zkmBatchDownloader - Error: " + `data`)
		writeTracebackAsString(20)

	return

if __name__ == "__main__":
	try:
		argsError = 0
		arg1, val1 = string.split(sys.argv[1], "=")
		arg2, val2 = string.split(sys.argv[2], "=")
		zopeHost = string.strip(val1)
		zopePort = int(string.strip(val2))
	except:
		argsError = 1

	if argsError or (arg1 != "zopeHost") or (arg2 != "zopePort"):
		print "Usage: python zkmBatchDownloader.py zopeHost=hostname zopePort=nnnn\n"
		print "Invalid command arguments. Please see usage note above."
		sys.exit(-1)

	serve(zopeHost, zopePort)
