#zkmProxy external method module

from zkm.zkmRequest import ZkmRequest
from zkm.zkmCommand import ZkmCommand
from zkm.zkmSocket import ZkmSocket

from zkm import zkmUrl
from zkm import zkmFile
from zkm.zkmHtml import formatError, RETRY_OR_SEE_LOG
from zkm.zkmTrace import writeTrace, writeError, writeTracebackAsString

import urllib
import string
import time
import os

def createTempFile(tempFolder, userName, htmlPageString):
	try:
		filePathName = zkmFile.createTempFileName(tempFolder, userName)
		f = open(filePathName, "w")
		f.write(htmlPageString)
		f.close()
	except:
		try:
			os.remove(filePathName)
		except:
			pass
		raise

	return filePathName

def createSavePageCommand(aZkmRequest, filePathName):
	userName = aZkmRequest.userName
	url = aZkmRequest.urlAfterOpen
	graphicsCaching = aZkmRequest.graphicsCaching

	dateAndTime = `time.time()`
	aZkmCommand = ZkmCommand(command="save", userName=userName, url=url,
							 filePathName=filePathName, graphicsCaching=graphicsCaching,
							 dateAndTime=dateAndTime)
	return aZkmCommand

def saveAndIndexPage(command, serverName, serverPort):
	aZkmSocket = ZkmSocket(command, serverName, serverPort)

	if not aZkmSocket.sendCommandNoResponse():
		raise Exception, "Error sending 'save' command."

	return

def createProxifyCommand(aZkmRequest, filePathName):
	url = aZkmRequest.urlAfterOpen
	graphicsCaching = aZkmRequest.graphicsCaching

	aZkmCommand = ZkmCommand(command="proxify", url=url, filePathName=filePathName,
							 graphicsCaching=graphicsCaching)
	return aZkmCommand

def proxifyPage(command, serverName, serverPort):
	try:
		aZkmSocket = ZkmSocket(command, serverName, serverPort)

		response = aZkmSocket.sendCommandGetResponse()

		if not response[0]:
			writeError("Error in 'proxify' command.", command=command)
		else:
			result = response[1]
			return result

	except Exception, data:
		writeError("Error retrieving page: " + `data`)
		writeTrackbackAsString(20)

	result = formatError("Error 'proxifying' page. " + RETRY_OR_SEE_LOG)

	return result

def zkmProxy(self, REQUEST, AUTHENTICATED_USER):
	try:
		status = "determining proxy request"

		aZkmRequest = ZkmRequest(REQUEST, AUTHENTICATED_USER)

		#aZkmRequest.writeTrace()

		status = "opening URL"

		urlFileObj = urllib.urlopen(aZkmRequest.urlToOpen)

		list = urlFileObj.readlines()
		htmlPageString = ""

		for item in list:
			htmlPageString = htmlPageString + item

		status = "adjusting URL if necessary due to redirection"

		aZkmRequest.urlAfterOpen = urlFileObj.geturl()
		urlFileObj.close()
		aZkmRequest.zkmBase = zkmUrl.getUrlBaseFromFullUrl(aZkmRequest.urlAfterOpen)

		status = "determining system configuration information"

		tempFolder = self.aq_parent["tempFolder"]
		serverPort = self.aq_parent["serverPort"]

		status = "creating HTML page temp file"

		filePathName = createTempFile(tempFolder, aZkmRequest.userName, htmlPageString)

		status = "converting HTML for return to proxy"

		command = createProxifyCommand(aZkmRequest, filePathName)

		htmlPageString = proxifyPage(command, aZkmRequest.serverName, serverPort)

		status = "saving and indexing HTML page"

		command = createSavePageCommand(aZkmRequest, filePathName)

		saveAndIndexPage(command, aZkmRequest.serverName, serverPort)

		return htmlPageString

	except Exception, data:
		writeError("Error " + status + ": " + `data`)
		writeTracebackAsString(20)

		return formatError("Error " + status + ". " + RETRY_OR_SEE_LOG)
