#zkmSearch and zkmRetrieve external methods module

from zkm.zkmRequest import ZkmRequest
from zkm.zkmCommand import ZkmCommand
from zkm.zkmSocket import ZkmSocket
from zkm.zkmHtml import formatError, RETRY_OR_SEE_LOG

from zkm.zkmTrace import writeTrace, writeError, writeTracebackAsString

import string
import time

def createRetrieveCommand(userName,pageId):
	aZkmCommand = ZkmCommand(command="retrieve", userName=userName,
							 pageId=pageId, graphicsCaching=0)
	return aZkmCommand

def zkmRetrieve(self, REQUEST, AUTHENTICATED_USER):
	errorMsg = "Error in 'retrieve' command."
	request = REQUEST
	pageId = request["pageId"]
	httpHost = request["HTTP_HOST"]
	serverName = string.split(httpHost, ":")[0]
	serverPort = self.aq_parent["serverPort"]

	userName = `AUTHENTICATED_USER`

	try:
		command = createRetrieveCommand(userName, pageId)
		aZkmSocket = ZkmSocket(command, serverName, serverPort)

		response = aZkmSocket.sendCommandGetResponse()

		if not response[0]:
			writeError(errorMsg, command=command)
		else:
			result = response[1]
			return result

	except Exception, data:
		writeError(errorMsg + `data`)
		writeTrackbackAsString(20)

	result = formatError(errorMsg + RETRY_OR_SEE_LOG)

	return result

def createRetrieveFromCacheCommand(userName,url):
	aZkmCommand = ZkmCommand(command="retrieveFromCache", userName=userName,
							 url=url, graphicsCaching=0)
	return aZkmCommand

def zkmCache(self, REQUEST, AUTHENTICATED_USER):
	errorMsg = "Error in 'retrieve from cache' command."
	aZkmRequest = ZkmRequest(REQUEST, AUTHENTICATED_USER)
	userName = aZkmRequest.userName
	serverName = aZkmRequest.serverName
	url = aZkmRequest.urlToOpen
	serverPort = self.aq_parent["serverPort"]

	try:
		command = createRetrieveFromCacheCommand(userName, url)
		aZkmSocket = ZkmSocket(command, serverName, serverPort)

		response = aZkmSocket.sendCommandGetResponse()

		if not response[0]:
			writeError(errorMsg, command=command)
		else:
			result = response[1]
			return result

	except Exception, data:
		writeError(errorMsg + `data`)
		writeTrackbackAsString(20)

	result = formatError(errorMsg + "<br>" + url + "<br>" + RETRY_OR_SEE_LOG)

	return result

def createQueryCommand(userName, query):
	aZkmCommand = ZkmCommand(command="query", userName=userName, query=query)
	return aZkmCommand

def zkmSearch(self, REQUEST, AUTHENTICATED_USER):
	errorMsg = "Error in 'query' command."
	request = REQUEST
	query = request["query"]
	httpHost = request["HTTP_HOST"]
	serverName = string.split(httpHost, ":")[0]
	serverPort = self.aq_parent["serverPort"]

	userName = `AUTHENTICATED_USER`

	try:
		command = createQueryCommand(userName, query)
		aZkmSocket = ZkmSocket(command, serverName, serverPort)

		response = aZkmSocket.sendCommandGetResponse()

		if not response[0]:
			writeError(errorMsg, command=command)
		else:
			result = response[1]
			return result

	except Exception, data:
		writeError(errorMsg + `data`)
		writeTrackbackAsString(20)

	result = formatError(errorMsg + RETRY_OR_SEE_LOG)

	return result
