#zkmHtml module, zkm package

from zkm import zkmFile
from zkm import zkmUrl
from zkm.zkmTrace import writeError, writeTrace

from Porter import stem

#from adzapper import adzapper_engine
#from zaplet import zaplet_engine

import re
import string
import sys

urlBase = ""
httpServerPort = 0
httpServerName = ""
graphicsCaching = 0
binariesFolder = ""
adzapperCombinedEngine = None

RETRY_OR_SEE_LOG = "Please retry or see error log for details."

nonHtmlDocExtensions = ["aif", "aiff", "au", "avi", "bmp", "doc", "gif", "gtar", "gz", "gzip",
			   			"hqx", "jfif", "jpeg", "jpe", "jpg", "mp2", "mp3",
					   	"mov", "movie", "mpeg", "mpg", "mpe", "png",
			   			"pdf", "ps", "qt", "ram", "rm", "ra", "sit", "snd",
			   			"tar", "tgz", "tiff", "tif", "txt", "wav", "xbm", "xls", "Z", "zip"]

htmlDocExtensions = ["asp", "htm", "html", "jsp"]	#check Doug's list--in email--there may be more!

javascriptEvents = ["onclick", "oncontextmenu", "ondblclick", "ondrag", "ondragend",
					"ondragenter", "ondragleave", "ondragover", "ondragstart", "ondrop",
					"onhelp", "onkeydown", "onkeypress", "onkeyup", "onmousedown",
					"onmousemove", "onmouseout", "onmouseover", "onmouseup",
					"onpropertychange", "onreadystatechange"]

#regular expression compiled patterns
anchorPat = re.compile(r'(?P<anchorTagBegin><[^>]+?\b(href|usemap)\s*?=\s*?["\']?)(?P<anchorUrl>[^"\' >]+)',
					   re.IGNORECASE | re.DOTALL)
beginScriptPat = re.compile(r'<script>(?P<script>.+?)</script>', re.IGNORECASE | re.DOTALL)
beginTagPat = re.compile(r'(?P<examine><.+?>)', re.IGNORECASE | re.DOTALL)
bodyBgrndPat = re.compile(r'(?P<bodyTagBegin><body[^>]+?\bbackground\s*?=\s*?["\']?)(?P<bgrndUrl>[^"\' >]+)',
						  re.IGNORECASE | re.DOTALL)
commentPat = re.compile(r'<!--(?P<keep>.*?)-->', re.IGNORECASE | re.DOTALL)
endTagPat = re.compile(r'(?P<eliminate></.*?>)', re.IGNORECASE | re.DOTALL)
extScriptPat = re.compile(r'(?P<extScriptPatBegin><script[^>]+?\bsrc\s*?=\s*?["\']?)(?P<extScriptUrl>[^<]*?</script>)',
						  re.IGNORECASE | re.DOTALL)
extraWhitePat = re.compile(r'(?P<extraWhite>\s\s+)', re.IGNORECASE | re.DOTALL)
formActionPat = re.compile(r'(?P<formTagBegin><form[^>]+?\baction\s*?=\s*?["\']?)(?P<actionUrl>[^"\' >]+)(?P<rest>.*?>)(?P<formStuff>.*?)(?P<formEnd></form>)',
						   re.IGNORECASE | re.DOTALL)
headBeginTagPat = re.compile(r'<html[^>]*?>\s*?<head[^>]*?>', re.IGNORECASE | re.DOTALL)
imageObjectSrcPropertyPat = re.compile(r'(?P<srcTagBegin>\.src\s*?=\s*?["\'])(?P<srcUrl>[^"\' >]+)',
									   re.IGNORECASE | re.DOTALL)
imgPat = re.compile(r'(?:<img[^>]+?\b(?:low|dyn)?src\s*?=\s*?["\']?)([^"\' >]+)',
					   re.IGNORECASE | re.DOTALL)
javascriptEventPat = re.compile(r'(?P<jsEventTagBegin> on[a-zA-Z]+?\s*?=\s*?")(?P<jsEventCode>[^"]+)',
								re.IGNORECASE | re.DOTALL)
metaRefreshPat = re.compile(r'(?P<metaTagBegin><meta[^>]+?\burl\s*?=\s*?)(?P<metaRefreshUrl>[^"\' >]+)',
					   re.IGNORECASE | re.DOTALL)
nbspPat = re.compile(r'&nbsp;', re.IGNORECASE | re.DOTALL)
pdfPat = re.compile(r'(?:<a[^>]+?\bhref\s*?=\s*?["\']?)([^"\' ]+?\.pdf)',
					   re.IGNORECASE | re.DOTALL)
proxyUrlPat = re.compile(r'(?P<proxyUrl>/zkm/bin/zkmProxy)', re.IGNORECASE | re.DOTALL)
srcPat = re.compile(r'(?P<srcTagBegin><[^>]+?[^.]\b(low|dyn)?src\s*?=\s*?["\']?)(?P<srcUrl>[^"\' >]+)',
					   re.IGNORECASE | re.DOTALL)
stringPat = re.compile(r'\'(?P<stringContents>.*?)\'',
					   re.IGNORECASE | re.DOTALL)
stylePat = re.compile(r'(?P<eliminate><style.*?</style.*?>)', re.IGNORECASE | re.DOTALL)
zkmUrlPat = re.compile(r'(?:<meta name="zkmUrl" content=")([^"]+)',
					   re.IGNORECASE | re.DOTALL)

#use stopWordsMerger.py utility to re-create the following dictionary,
#if necessary. Yes, all values in this dictionary are None. The dictionary
#is only used to see if the keys exist.
stopwords = {
	"A" : None,
	"A'S" : None,
	"ABLE" : None,
	"ABOUT" : None,
	"ABOVE" : None,
	"ACCORDING" : None,
	"ACCORDINGLY" : None,
	"ACROSS" : None,
	"ACTUALLY" : None,
	"AFTER" : None,
	"AFTERWARDS" : None,
	"AGAIN" : None,
	"AGAINST" : None,
	"AIN'T" : None,
	"ALL" : None,
	"ALLOW" : None,
	"ALLOWS" : None,
	"ALMOST" : None,
	"ALONE" : None,
	"ALONG" : None,
	"ALREADY" : None,
	"ALSO" : None,
	"ALTHOUGH" : None,
	"ALWAYS" : None,
	"AM" : None,
	"AMONG" : None,
	"AMONGST" : None,
	"AMOUNGST" : None,
	"AMOUNT" : None,
	"AN" : None,
	"AND" : None,
	"ANOTHER" : None,
	"ANY" : None,
	"ANYBODY" : None,
	"ANYHOW" : None,
	"ANYONE" : None,
	"ANYTHING" : None,
	"ANYWAY" : None,
	"ANYWAYS" : None,
	"ANYWHERE" : None,
	"APART" : None,
	"APPEAR" : None,
	"APPRECIATE" : None,
	"APPROPRIATE" : None,
	"ARE" : None,
	"AREN'T" : None,
	"AROUND" : None,
	"AS" : None,
	"ASIDE" : None,
	"ASK" : None,
	"ASKING" : None,
	"ASSOCIATED" : None,
	"AT" : None,
	"AVAILABLE" : None,
	"AWAY" : None,
	"AWFULLY" : None,
	"B" : None,
	"BACK" : None,
	"BE" : None,
	"BECAME" : None,
	"BECAUSE" : None,
	"BECOME" : None,
	"BECOMES" : None,
	"BECOMING" : None,
	"BEEN" : None,
	"BEFORE" : None,
	"BEFOREHAND" : None,
	"BEHIND" : None,
	"BEING" : None,
	"BELIEVE" : None,
	"BELOW" : None,
	"BESIDE" : None,
	"BESIDES" : None,
	"BEST" : None,
	"BETTER" : None,
	"BETWEEN" : None,
	"BEYOND" : None,
	"BILL" : None,
	"BOTH" : None,
	"BOTTOM" : None,
	"BRIEF" : None,
	"BUT" : None,
	"BY" : None,
	"C" : None,
	"C'MON" : None,
	"C'S" : None,
	"CALL" : None,
	"CAME" : None,
	"CAN" : None,
	"CAN'T" : None,
	"CANNOT" : None,
	"CANT" : None,
	"CAUSE" : None,
	"CAUSES" : None,
	"CERTAIN" : None,
	"CERTAINLY" : None,
	"CHANGES" : None,
	"CLEARLY" : None,
	"CO" : None,
	"COM" : None,
	"COME" : None,
	"COMES" : None,
	"COMPUTER" : None,
	"CON" : None,
	"CONCERNING" : None,
	"CONSEQUENTLY" : None,
	"CONSIDER" : None,
	"CONSIDERING" : None,
	"CONTAIN" : None,
	"CONTAINING" : None,
	"CONTAINS" : None,
	"CORRESPONDING" : None,
	"COULD" : None,
	"COULDN'T" : None,
	"COULDNT" : None,
	"COURSE" : None,
	"CRY" : None,
	"CURRENTLY" : None,
	"D" : None,
	"DE" : None,
	"DEFINITELY" : None,
	"DESCRIBE" : None,
	"DESCRIBED" : None,
	"DESPITE" : None,
	"DETAIL" : None,
	"DID" : None,
	"DIDN'T" : None,
	"DIFFERENT" : None,
	"DO" : None,
	"DOES" : None,
	"DOESN'T" : None,
	"DOING" : None,
	"DON'T" : None,
	"DONE" : None,
	"DOWN" : None,
	"DOWNWARDS" : None,
	"DUE" : None,
	"DURING" : None,
	"E" : None,
	"EACH" : None,
	"EDU" : None,
	"EG" : None,
	"EIGHT" : None,
	"EITHER" : None,
	"ELEVEN" : None,
	"ELSE" : None,
	"ELSEWHERE" : None,
	"EMPTY" : None,
	"ENOUGH" : None,
	"ENTIRELY" : None,
	"ESPECIALLY" : None,
	"ET" : None,
	"ETC" : None,
	"EVEN" : None,
	"EVER" : None,
	"EVERY" : None,
	"EVERYBODY" : None,
	"EVERYONE" : None,
	"EVERYTHING" : None,
	"EVERYWHERE" : None,
	"EX" : None,
	"EXACTLY" : None,
	"EXAMPLE" : None,
	"EXCEPT" : None,
	"F" : None,
	"FAR" : None,
	"FEW" : None,
	"FIFTEEN" : None,
	"FIFTH" : None,
	"FIFY" : None,
	"FILL" : None,
	"FIND" : None,
	"FIRE" : None,
	"FIRST" : None,
	"FIVE" : None,
	"FOLLOWED" : None,
	"FOLLOWING" : None,
	"FOLLOWS" : None,
	"FOR" : None,
	"FORMER" : None,
	"FORMERLY" : None,
	"FORTH" : None,
	"FORTY" : None,
	"FOUND" : None,
	"FOUR" : None,
	"FROM" : None,
	"FRONT" : None,
	"FULL" : None,
	"FURTHER" : None,
	"FURTHERMORE" : None,
	"G" : None,
	"GET" : None,
	"GETS" : None,
	"GETTING" : None,
	"GIVE" : None,
	"GIVEN" : None,
	"GIVES" : None,
	"GO" : None,
	"GOES" : None,
	"GOING" : None,
	"GONE" : None,
	"GOT" : None,
	"GOTTEN" : None,
	"GREETINGS" : None,
	"H" : None,
	"HAD" : None,
	"HADN'T" : None,
	"HAPPENS" : None,
	"HARDLY" : None,
	"HAS" : None,
	"HASN'T" : None,
	"HASNT" : None,
	"HAVE" : None,
	"HAVEN'T" : None,
	"HAVING" : None,
	"HE" : None,
	"HE'S" : None,
	"HELLO" : None,
	"HELP" : None,
	"HENCE" : None,
	"HER" : None,
	"HERE" : None,
	"HERE'S" : None,
	"HEREAFTER" : None,
	"HEREBY" : None,
	"HEREIN" : None,
	"HEREUPON" : None,
	"HERS" : None,
	"HERSELF" : None,
	"HI" : None,
	"HIM" : None,
	"HIMSELF" : None,
	"HIS" : None,
	"HITHER" : None,
	"HOPEFULLY" : None,
	"HOW" : None,
	"HOWBEIT" : None,
	"HOWEVER" : None,
	"HUNDRED" : None,
	"I" : None,
	"I'D" : None,
	"I'LL" : None,
	"I'M" : None,
	"I'VE" : None,
	"IE" : None,
	"IF" : None,
	"IGNORED" : None,
	"IMMEDIATE" : None,
	"IN" : None,
	"INASMUCH" : None,
	"INC" : None,
	"INDEED" : None,
	"INDICATE" : None,
	"INDICATED" : None,
	"INDICATES" : None,
	"INNER" : None,
	"INSOFAR" : None,
	"INSTEAD" : None,
	"INTEREST" : None,
	"INTO" : None,
	"INWARD" : None,
	"IS" : None,
	"ISN'T" : None,
	"IT" : None,
	"IT'D" : None,
	"IT'LL" : None,
	"IT'S" : None,
	"ITS" : None,
	"ITSELF" : None,
	"J" : None,
	"JUST" : None,
	"K" : None,
	"KEEP" : None,
	"KEEPS" : None,
	"KEPT" : None,
	"KNOW" : None,
	"KNOWN" : None,
	"KNOWS" : None,
	"L" : None,
	"LAST" : None,
	"LATELY" : None,
	"LATER" : None,
	"LATTER" : None,
	"LATTERLY" : None,
	"LEAST" : None,
	"LESS" : None,
	"LEST" : None,
	"LET" : None,
	"LET'S" : None,
	"LIKE" : None,
	"LIKED" : None,
	"LIKELY" : None,
	"LITTLE" : None,
	"LOOK" : None,
	"LOOKING" : None,
	"LOOKS" : None,
	"LTD" : None,
	"M" : None,
	"MADE" : None,
	"MAINLY" : None,
	"MANY" : None,
	"MAY" : None,
	"MAYBE" : None,
	"ME" : None,
	"MEAN" : None,
	"MEANWHILE" : None,
	"MERELY" : None,
	"MIGHT" : None,
	"MILL" : None,
	"MINE" : None,
	"MORE" : None,
	"MOREOVER" : None,
	"MOST" : None,
	"MOSTLY" : None,
	"MOVE" : None,
	"MUCH" : None,
	"MUST" : None,
	"MY" : None,
	"MYSELF" : None,
	"N" : None,
	"NAME" : None,
	"NAMELY" : None,
	"ND" : None,
	"NEAR" : None,
	"NEARLY" : None,
	"NECESSARY" : None,
	"NEED" : None,
	"NEEDS" : None,
	"NEITHER" : None,
	"NEVER" : None,
	"NEVERTHELESS" : None,
	"NEW" : None,
	"NEXT" : None,
	"NINE" : None,
	"NO" : None,
	"NOBODY" : None,
	"NON" : None,
	"NONE" : None,
	"NOONE" : None,
	"NOR" : None,
	"NORMALLY" : None,
	"NOT" : None,
	"NOTHING" : None,
	"NOVEL" : None,
	"NOW" : None,
	"NOWHERE" : None,
	"O" : None,
	"OBVIOUSLY" : None,
	"OF" : None,
	"OFF" : None,
	"OFTEN" : None,
	"OH" : None,
	"OK" : None,
	"OKAY" : None,
	"OLD" : None,
	"ON" : None,
	"ONCE" : None,
	"ONE" : None,
	"ONES" : None,
	"ONLY" : None,
	"ONTO" : None,
	"OR" : None,
	"OTHER" : None,
	"OTHERS" : None,
	"OTHERWISE" : None,
	"OUGHT" : None,
	"OUR" : None,
	"OURS" : None,
	"OURSELVES" : None,
	"OUT" : None,
	"OUTSIDE" : None,
	"OVER" : None,
	"OVERALL" : None,
	"OWN" : None,
	"P" : None,
	"PART" : None,
	"PARTICULAR" : None,
	"PARTICULARLY" : None,
	"PER" : None,
	"PERHAPS" : None,
	"PLACED" : None,
	"PLEASE" : None,
	"PLUS" : None,
	"POSSIBLE" : None,
	"PRESUMABLY" : None,
	"PROBABLY" : None,
	"PROVIDES" : None,
	"PUT" : None,
	"Q" : None,
	"QUE" : None,
	"QUITE" : None,
	"QV" : None,
	"R" : None,
	"RATHER" : None,
	"RD" : None,
	"RE" : None,
	"REALLY" : None,
	"REASONABLY" : None,
	"REGARDING" : None,
	"REGARDLESS" : None,
	"REGARDS" : None,
	"RELATIVELY" : None,
	"RESPECTIVELY" : None,
	"RIGHT" : None,
	"S" : None,
	"SAID" : None,
	"SAME" : None,
	"SAW" : None,
	"SAY" : None,
	"SAYING" : None,
	"SAYS" : None,
	"SECOND" : None,
	"SECONDLY" : None,
	"SEE" : None,
	"SEEING" : None,
	"SEEM" : None,
	"SEEMED" : None,
	"SEEMING" : None,
	"SEEMS" : None,
	"SEEN" : None,
	"SELF" : None,
	"SELVES" : None,
	"SENSIBLE" : None,
	"SENT" : None,
	"SERIOUS" : None,
	"SERIOUSLY" : None,
	"SEVEN" : None,
	"SEVERAL" : None,
	"SHALL" : None,
	"SHE" : None,
	"SHOULD" : None,
	"SHOULDN'T" : None,
	"SHOW" : None,
	"SIDE" : None,
	"SINCE" : None,
	"SINCERE" : None,
	"SIX" : None,
	"SIXTY" : None,
	"SO" : None,
	"SOME" : None,
	"SOMEBODY" : None,
	"SOMEHOW" : None,
	"SOMEONE" : None,
	"SOMETHING" : None,
	"SOMETIME" : None,
	"SOMETIMES" : None,
	"SOMEWHAT" : None,
	"SOMEWHERE" : None,
	"SOON" : None,
	"SORRY" : None,
	"SPECIFIED" : None,
	"SPECIFY" : None,
	"SPECIFYING" : None,
	"STILL" : None,
	"SUB" : None,
	"SUCH" : None,
	"SUP" : None,
	"SURE" : None,
	"SYSTEM" : None,
	"T" : None,
	"T'S" : None,
	"TAKE" : None,
	"TAKEN" : None,
	"TELL" : None,
	"TEN" : None,
	"TENDS" : None,
	"TH" : None,
	"THAN" : None,
	"THANK" : None,
	"THANKS" : None,
	"THANX" : None,
	"THAT" : None,
	"THAT'S" : None,
	"THATS" : None,
	"THE" : None,
	"THEIR" : None,
	"THEIRS" : None,
	"THEM" : None,
	"THEMSELVES" : None,
	"THEN" : None,
	"THENCE" : None,
	"THERE" : None,
	"THERE'S" : None,
	"THEREAFTER" : None,
	"THEREBY" : None,
	"THEREFORE" : None,
	"THEREIN" : None,
	"THERES" : None,
	"THEREUPON" : None,
	"THESE" : None,
	"THEY" : None,
	"THEY'D" : None,
	"THEY'LL" : None,
	"THEY'RE" : None,
	"THEY'VE" : None,
	"THICK" : None,
	"THIN" : None,
	"THINK" : None,
	"THIRD" : None,
	"THIS" : None,
	"THOROUGH" : None,
	"THOROUGHLY" : None,
	"THOSE" : None,
	"THOUGH" : None,
	"THREE" : None,
	"THROUGH" : None,
	"THROUGHOUT" : None,
	"THRU" : None,
	"THUS" : None,
	"TO" : None,
	"TOGETHER" : None,
	"TOO" : None,
	"TOOK" : None,
	"TOP" : None,
	"TOWARD" : None,
	"TOWARDS" : None,
	"TRIED" : None,
	"TRIES" : None,
	"TRULY" : None,
	"TRY" : None,
	"TRYING" : None,
	"TWELVE" : None,
	"TWENTY" : None,
	"TWICE" : None,
	"TWO" : None,
	"U" : None,
	"UN" : None,
	"UNDER" : None,
	"UNFORTUNATELY" : None,
	"UNLESS" : None,
	"UNLIKELY" : None,
	"UNTIL" : None,
	"UNTO" : None,
	"UP" : None,
	"UPON" : None,
	"US" : None,
	"USE" : None,
	"USED" : None,
	"USEFUL" : None,
	"USES" : None,
	"USING" : None,
	"USUALLY" : None,
	"UUCP" : None,
	"V" : None,
	"VALUE" : None,
	"VARIOUS" : None,
	"VERY" : None,
	"VIA" : None,
	"VIZ" : None,
	"VS" : None,
	"W" : None,
	"WANT" : None,
	"WANTS" : None,
	"WAS" : None,
	"WASN'T" : None,
	"WAY" : None,
	"WE" : None,
	"WE'D" : None,
	"WE'LL" : None,
	"WE'RE" : None,
	"WE'VE" : None,
	"WELCOME" : None,
	"WELL" : None,
	"WENT" : None,
	"WERE" : None,
	"WEREN'T" : None,
	"WHAT" : None,
	"WHAT'S" : None,
	"WHATEVER" : None,
	"WHEN" : None,
	"WHENCE" : None,
	"WHENEVER" : None,
	"WHERE" : None,
	"WHERE'S" : None,
	"WHEREAFTER" : None,
	"WHEREAS" : None,
	"WHEREBY" : None,
	"WHEREIN" : None,
	"WHEREUPON" : None,
	"WHEREVER" : None,
	"WHETHER" : None,
	"WHICH" : None,
	"WHILE" : None,
	"WHITHER" : None,
	"WHO" : None,
	"WHO'S" : None,
	"WHOEVER" : None,
	"WHOLE" : None,
	"WHOM" : None,
	"WHOSE" : None,
	"WHY" : None,
	"WILL" : None,
	"WILLING" : None,
	"WISH" : None,
	"WITH" : None,
	"WITHIN" : None,
	"WITHOUT" : None,
	"WON'T" : None,
	"WONDER" : None,
	"WOULD" : None,
	"WOULDN'T" : None,
	"X" : None,
	"Y" : None,
	"YES" : None,
	"YET" : None,
	"YOU" : None,
	"YOU'D" : None,
	"YOU'LL" : None,
	"YOU'RE" : None,
	"YOU'VE" : None,
	"YOUR" : None,
	"YOURS" : None,
	"YOURSELF" : None,
	"YOURSELVES" : None,
	"Z" : None,
	"ZERO" : None
	}

#The following dictionary entries tell whether there is useful "domain" text
#(for indexing) in the ATTRIBUTES of the tags. They do NOT refer to info. in
#between the start and the end tag if applicable.
tagAttributesToSave = {
	"A" : 			("HREF", ".SRC"),
	"ABBR" : 		("TITLE",),
	"ACRONYM" : 	("TITLE",),
	"ADDRESS" : 	None,
	"APPLET" : 		("CODE",),
	"AREA" : 		("HREF","ALT",),
	"B" : 			None,
	"BASE" : 		("HREF",),
	"BASEFONT" : 	None,
	"BDO" : 		None,
	"BGSOUND" : 	("SRC",),
	"BIG" : 		None,
	"BLINK" : 		None,
	"BLOCKQUOTE" : 	("CITE",),
	"BODY" : 		("BACKGROUND",),
	"BR" : 			None,
	"BUTTON" : 		None,
	"CAPTION" : 	None,
	"CENTER" : 		None,
	"CITE" : 		None,
	"CODE" : 		None,
	"COL" : 		None,
	"COLGROUP" : 	None,
	"DD" : 			None,
	"DEL" : 		None,
	"DIV" : 		None,
	"!DOCTYPE" :	None,
	"DL" : 			None,
	"DT" : 			None,
	"EM" : 			None,
	"EMBED" : 		("SRC",),
	"FIELDSET" : 	None,
	"FONT" : 		None,
	"FORM" : 		("ACTION",),
	"FRAME" : 		("SRC",),
	"FRAMESET" : 	None,
	"H1" : 			None,
	"H2" : 			None,
	"H3" : 			None,
	"H4" : 			None,
	"H5" : 			None,
	"H6" : 			None,
	"HEAD" : 		None,
	"HR" : 			None,
	"HTML" : 		None,
	"I" : 			None,
	"IFRAME" : 		("SRC",),
	"IMG" : 		("SRC","ALT","LOWSRC","DYNSRC"),
	"INPUT" : 		("SRC",),
	"INS" : 		None,
	"KBD" : 		None,
	"LABEL" : 		None,
	"LAYER" : 		("SRC","BACKGROUND"),
	"LEGEND" : 		None,
	"LI" : 			None,
	"LINK" : 		None,
	"MAP" : 		None,
	"MARQUEE" : 	None,
	"MENU" : 		None,
	"META" : 		("URL","CONTENT",),
	"MULTICOL" : 	None,
	"NOBR" : 		None,
	"NOEMBED" : 	None,
	"NOFRAMES" : 	None,
	"NOLAYER" : 	None,
	"NOSCRIPT" : 	None,
	"OBJECT" : 		("CLASSID","DATA",),
	"OL" : 			None,
	"OPTGROUP" : 	("LABEL",),
	"OPTION" : 		("VALUE",),
	"P" : 			None,
	"PLAINTEXT" : 	None,
	"PRE" : 		None,
	"Q" : 			None,
	"S" : 			None,
	"SAMP" : 		None,
	"SCRIPT" : 		None,	#scripts are handled specially
	"SELECT" : 		None,
	"SMALL" : 		None,
	"SPACER" : 		None,
	"SPAN" : 		None,
	"STRIKE" : 		None,
	"STRONG" : 		None,
	"STYLE" : 		None,	#styles are handled specially
	"SUB" : 		None,
	"SUP" : 		None,
	"TABLE" : 		None,
	"TBODY" : 		None,
	"TD" : 			None,
	"TEXTAREA" : 	None,
	"TFOOT" : 		None,
	"TH" : 			None,
	"THEAD" : 		None,
	"TITLE" : 		None,
	"TR" : 			None,
	"TT" : 			None,
	"U" : 			None,
	"UL" : 			None,
	"WBR" : 		None,
	"XMP" : 		None
	}

#a dictionary for saving beginning tag attribute useful domain text for
#appending to the normal visible text
beginTagText = {}

ADZAPPER_CONFIG_FILE = 'adzapper.conf'

def eliminateReplaceFunction(match):
	#always return a string of one space/blank char
	return " "

def keepReplaceFunction(match):
	#always return what was found
	s = match.group("keep")
	return s

def saveBeginTagTextReplaceFunction(match):
	#This function attempts to pick potentially useful domain info. out of beginning
	#HTML tags. It works by adding these values--to be used later--to the global
	#dictionary beginTagText. Certain beginning HTML tags are skipped because they
	#normally don't contain useful domain information.

	global beginTagText

	s = match.group("examine")
	tagWord = string.split(s, ">")
	tagWord = string.split(tagWord[0], " ")
	tagWord = string.strip(tagWord[0])
	tagWord = string.upper(tagWord[1:])

	tagWordEntry = tagAttributesToSave.get(tagWord, "NoEntry")

	if tagWordEntry == "NoEntry":
		beginTagText[s] = None
		return " "

	elif tagWordEntry == None:
		return " "

	else:
		#IMPORTANT FUTURE TO DO: Rewrite this part using better variable names.
		#Used simple names originally because it's kind of hard to describe what
		#is being done here.

		#split text based on = signs
		a = string.split(s, "=")
		b = []
		for c in a:
			#split text again based on blank spaces
			d = string.split(c)
			for e in d:
				f = ""
				for ch in e:
					#screen out unwanted special characters
					if ch not in ['<', "'", '"', '>']:
						#break up URLs into words using spaces, etc.
						if not ((ch in string.digits) or (ch in string.letters) or (ch in "%")):
							ch = ' '
						f = f + ch
				#add cleaned up word/phrase to new list--list contains clean attributes and their
				#values following
				f = string.strip(f)
				if len(f) > 0:
					b.append(f)

		#build domain text string by picking out attribute values we're interested in
		s = ""
		for i in range(0, len(b)-1):
			if string.upper(b[i]) in tagWordEntry:
				#s = s + " " + b[i+1]
				for g in string.split(b[i+1]):
					beginTagText[g] = None

		return " "

def extraWhiteReplaceFunction(match):
	#always return 1 space/blank char
	return " "

def saveScriptStringsReplaceFunction(match):
	#process scripts--save all strings from them after converting non-word characters to space/blank;
	#throw everything else (besides strings) away
	s = match.group("script")
	savedStrings = ""
	currentString = ""
	stringStack = []

	for ch in s:
		if ch == '"' or chr == "'":
			if stringStack and (ch == stringStack[-1]):
				stringStack = stringStack[:-1]
				if not stringStack:
					savedStrings = savedStrings + " " + currentString
					currentString = ""
			else:
				stringStack.append(ch)
		else:
			if stringStack:
				if ((ch in string.digits) or (ch in string.letters) or (ch in "%")):
					currentString = currentString + ch
				else:
					currentString = currentString + ' '

	return savedStrings

def removeStopWordsAndPunctuation(s):
	list = string.split(s, " ")
	newStr = ""
	nonDigits = string.letters + "%"
	digits = string.digits + string.whitespace

	for item in list:
		item = string.strip(item)
		word = string.upper(item)

		if not stopwords.has_key(word):
			newItem = ""
			isWord = 0

			for c in item:
				if c in nonDigits:
					newItem = newItem + c
					isWord = 1
				else:
					if c in digits:
						newItem = newItem + c
					else:
						newItem = newItem + " "
			if isWord:
				newStr = newStr + newItem + " "

	return newStr

def removeHtml(fileName):
	global beginTagText

	s = zkmFile.readFileIntoString(fileName)

	#init to empty dictionary
	beginTagText = {}

	#get rid of style instructions
	s = stylePat.sub(eliminateReplaceFunction, s)

	#keep comments, but get rid of their beginning and ending tags; don't worry about comments
	#in a script because the whole script can be a comment
	s = commentPat.sub(keepReplaceFunction, s)

	#process scripts--save all strings from them; throw everything else away; delimit by
	# *ss* and *se* markers for ease in spotting in database
	s = beginScriptPat.sub(saveScriptStringsReplaceFunction, s)

	#get rid of end tags--period!
	s = endTagPat.sub(eliminateReplaceFunction, s)

	#get rid of &nbsp;
	s = nbspPat.sub(eliminateReplaceFunction, s)

	#save beginning tag useful domain text in a dict (beginTagText) and add to end of text later
	s = beginTagPat.sub(saveBeginTagTextReplaceFunction, s)

 	#add any beginning tag attribute words that were saved at the end; sort them to eliminate/reduce
 	#any false proximity searches
 	btt = " "
 	bttkeys = beginTagText.keys()
 	bttkeys.sort()

 	for k in bttkeys:
 		btt = btt + k + " "

	#mark them for now so we can spot them in the database more easily
	s = s + " " + btt + " "

	#get rid of stop words
	s = removeStopWordsAndPunctuation(s)

	#get rid of extra whitespace including blank lines
 	s = extraWhitePat.sub(extraWhiteReplaceFunction, s)

	#perform stemming on the words that are left
	listOfWords = string.split(s, " ")
 	listOfStemmedWords = stem(listOfWords)
 	s = ""

 	for w in listOfStemmedWords:
 		s = s + w + " "

	return s

def addMetaDataToHtmlString(s, url, dateAndTime, userName):
	match = headBeginTagPat.search(s)

	#if no <html><head> pattern, we will wrap what's there
	if not match:
		htmlWithMetaData = ("<html><head>"
							+ '<meta name="zkmUrl" content="' + url + '">'
							+ '<meta name="zkmDateAndTime" content="' + dateAndTime + '">'
							+ '<meta name="zkmUserName" content="' + userName + '">\n'
							+ '</head>' + s + '</html>')
	else:
		endHeadBegin = match.end()
		htmlWithMetaData = (s[:endHeadBegin]
							+ '<meta name="zkmUrl" content="' + url + '">'
							+ '<meta name="zkmDateAndTime" content="' + dateAndTime + '">'
							+ '<meta name="zkmUserName" content="' + userName + '">\n'
							+ s[endHeadBegin:])
	return htmlWithMetaData

def findZkmUrl(s):
	#<meta name="zkmUrl" content="http://www.msn.com">

	tmatch = zkmUrlPat.search(s)
	if tmatch:
		url = tmatch.group(1)

	return url

def	findImageFileUrls(s, urlBase):
	#<img border="0" src="smiley.gif" alt="Smiley">

	tempList = imgPat.findall(s)
	tempList = filter(zkmUrl.isUrlForDisplayableImage, tempList)

	imageUrlsList = []
	imageUrlsDict = {}

	for url in tempList:
		imageUrlsDict[zkmUrl.constructAbsoluteUrl(urlBase, url)] = None

	tempList = bodyBgrndPat.findall(s)

	if tempList:
		url = tempList[0][1]
		if zkmUrl.isUrlForDisplayableImage(url):
			imageUrlsDict[zkmUrl.constructAbsoluteUrl(urlBase, url)] = None

	imageUrlsList = imageUrlsDict.keys()

	return imageUrlsList


def	findPdfFileUrls(s, urlBase):
	#<a href="www.w3schools.com" target="_blank>

	tempList = pdfPat.findall(s)
	tempList = filter(zkmUrl.isUrlForPdfFile, tempList)

	pdfUrlsList = []
	pdfUrlsDict = {}

	for url in tempList:
		pdfUrlsDict[zkmUrl.constructAbsoluteUrl(urlBase, url)] = None

	pdfUrlsList = pdfUrlsDict.keys()

	return pdfUrlsList

#class AdzapperCombinedEngine:
#	# holder object for adzapper_engine and zaplet_engine
#	def __init__(self):
#		self.adzapper = None
#		self.zaplet = None
#		return
#
#	def restart(self):
#		self.zaplet.__init__(self.adzapper.zaplet_dirs,self.adzapper.site_zaplet_dir)
#		return

#def makeCombinedEngine(configFileDirectory):
#	global adzapperCombinedEngine
#
#	if configFileDirectory[-1] != "\\":
#		configFileDirectory = configFileDirectory + "\\"
#
#	adzapper_config_file = configFileDirectory + ADZAPPER_CONFIG_FILE
#
#	# initialize the adzapper and zaplet engines
#	# make the holder object for the combined adzapper and zaplet engines
#	adzapperCombinedEngine = AdzapperCombinedEngine()
#
#	# initialize adzapper engine - for adzapper defaults, ACLs, etc.
#	adzapperCombinedEngine.adzapper = adzapper_engine.adzapper_engine(adzapper_config_file)
#
#	#port = adzapperCombinedEngine.adzapper.port--not needed!
#
#	# initialize zaplet engine - for blocking URLs
#	#
#	# zaplet_engine needs a list of dirs and the site zaplet dir
#	adzapperCombinedEngine.zaplet = zaplet_engine.zaplet_engine(adzapperCombinedEngine.adzapper.zaplet_dirs,adzapperCombinedEngine.adzapper.site_zaplet_dir)
#
#	return

def convertUrlIfForAdvertisement(absoluteUrl):
#	result = adzapperCombinedEngine.zaplet.urlCheckBlockAndFilter(absoluteUrl)
#
#	#IMPORTANT--DEBUGGING NEEDED--RESULT IS ALWAYS [0, 0, NONE]--ADZAPPING NOT WORKING!!!!
#	#print absoluteUrl
#	#print `result`
#	#print "-"*80
#
#	block = result[0]
#
#	if block != 0:
#		writeTrace("BLOCKED " + absoluteUrl + "\n")
#		print "BLOCKED " + `url` + "\n"
#		absoluteUrl = "/zkm/bin/dot_clear.gif"
#
	return absoluteUrl

def anchorForNonHtmlDoc(url):
	#see if extension of doc is in "special" list of non-HTML docs
	urlParts = string.split(url, ".")
	if len(urlParts) > 1:
		if string.lower(urlParts[-1]) in nonHtmlDocExtensions:
			return 1
	return 0

def anchorReplaceFunction(match):
	#try:
		anchorTagBegin = match.group("anchorTagBegin")
		anchorUrl = match.group("anchorUrl")

		#check for internal (same page) anchor--don't mess with them
		if anchorUrl[0] == "#":
			s = anchorTagBegin + anchorUrl

		#check for non-html anchors--don't mess with them either
		elif zkmUrl.isNonHtmlUrl(anchorUrl):
			s = anchorTagBegin + anchorUrl

		#check for anchor points to a target which we can't modify, such as a .PDF document
		elif anchorForNonHtmlDoc(anchorUrl):
			absoluteUrl = zkmUrl.constructAbsoluteUrl(urlBase, anchorUrl)

			if (len(binariesFolder) > 0) and (zkmUrl.isUrlForPdfFile(absoluteUrl)):
				filePathName = zkmUrl.filePathNameFromFullUrl(binariesFolder, absoluteUrl)
				binariesCacheFileUrl = zkmUrl.filePathNameToFileUrl(filePathName)
				s = anchorTagBegin + binariesCacheFileUrl
			else:
				s = anchorTagBegin + absoluteUrl
		else:
			anchorUrl = string.replace(anchorUrl, "?", "&")

			s = (anchorTagBegin +
				zkmUrl.constructProxyUrl(httpServerName, httpServerPort, urlBase,
										 anchorUrl, graphicsCaching))
		return s
	#except Exception, data:
	#	raise Exception, "Error in anchorReplaceFunction: anchorTagBegin=" + anchorTagBegin + " anchorUrl=" + anchorUrl

def metaRefreshReplaceFunction(match):
	metaTagBegin = match.group("metaTagBegin")
	metaRefreshUrl = match.group("metaRefreshUrl")
	metaRefreshUrl = string.replace(metaRefreshUrl, "?", "&")

	s = (metaTagBegin +
		zkmUrl.constructProxyUrl(httpServerName, httpServerPort, urlBase,
								 metaRefreshUrl, graphicsCaching))
	return s

def bgrndUrlReplaceFunction(match):
	bodyTagBegin = match.group("bodyTagBegin")
	bgrndUrl = match.group("bgrndUrl")
	absoluteUrl = zkmUrl.constructAbsoluteUrl(urlBase, bgrndUrl)
	absoluteUrl = convertUrlIfForAdvertisement(absoluteUrl)

	#redirect URL to the binaries cache if binariesFolder is not an empty string
	#(if we're in retrieve mode)
	if len(binariesFolder) > 0:
		filePathName = zkmUrl.filePathNameFromFullUrl(binariesFolder, absoluteUrl)
		binariesCacheFileUrl = zkmUrl.filePathNameToFileUrl(filePathName)
		s = bodyTagBegin + binariesCacheFileUrl
	else:
		s = bodyTagBegin + absoluteUrl
	return s

def srcReplaceFunction(match):
	global binariesFolder

	srcTagBegin = match.group("srcTagBegin")
	srcUrl = match.group("srcUrl")

	#frame tag src attribute--handle differently
	if string.lower(srcTagBegin[:2]) == "<f":
		srcUrl = string.replace(srcUrl, "?", "&")

		s = (srcTagBegin +
			zkmUrl.constructProxyUrl(httpServerName, httpServerPort, urlBase,
									 srcUrl, graphicsCaching))
	#non-frame tag src attribute
	else:
		absoluteUrl = zkmUrl.constructAbsoluteUrl(urlBase, srcUrl)
		absoluteUrl = convertUrlIfForAdvertisement(absoluteUrl)

		#check for img tag whose URL we need to redirect to the binaries cache if binariesFolder
		#is not an empty string (if we're in retrieve mode)
		if (string.lower(srcTagBegin[:4]) == "<img") and (len(binariesFolder) > 0):
			filePathName = zkmUrl.filePathNameFromFullUrl(binariesFolder, absoluteUrl)
			binariesCacheFileUrl = zkmUrl.filePathNameToFileUrl(filePathName)
			s = srcTagBegin + binariesCacheFileUrl
		else:
			s = srcTagBegin + absoluteUrl
	return s

def formActionUrlReplaceFunction(match):
	formTagBegin = match.group("formTagBegin")
	actionUrl = match.group("actionUrl")
	rest = match.group("rest")
	formStuff = match.group("formStuff")
	formEnd = match.group("formEnd")
	actionUrl = string.replace(actionUrl, "?", "&")

	if graphicsCaching:
		gCaching = '<input type=hidden name=zkmGc value="1">' + "\n"
	else:
		gCaching = ""

	s = (formTagBegin + "http://" + httpServerName + ":" + httpServerPort
		+ "/zkm/bin/zkmProxy" + rest + formStuff + "\n"
		+ '<input type=hidden name=zkmUrl value="'
		+ zkmUrl.constructAbsoluteUrl(urlBase, actionUrl) + '">' + "\n"
		+ gCaching
		+ formEnd)
	return s

def jsEventCodeReplaceFunction(match):
	s = match.group("stringContents")
	s = string.strip(s)
	i = string.rfind(s, ".")
	if i != -1 and len(s) > i:
		lastPart = string.lower(s[i+1:])
		if (lastPart in nonHtmlDocExtensions) or (lastPart in htmlDocExtensions):
			s = zkmUrl.constructAbsoluteUrl(urlBase, s)
			s = convertUrlIfForAdvertisement(s)
	return "'" + s + "'"

def jsEventReplaceFunction(match):
	global javascriptEvents
	jsEventTagBegin = match.group("jsEventTagBegin")
	jsEventCode = match.group("jsEventCode")

	#get event name
	jsEvent = string.split(jsEventTagBegin, "=")
	jsEvent = jsEvent[0]
	jsEvent = string.strip(jsEvent)

	#writeDebugInfo("jsEvent is " + jsEvent + " and jsEventCode is " + jsEventCode + "\n")

	if string.lower(jsEvent) in javascriptEvents:
		jsEventCode = stringPat.sub(jsEventCodeReplaceFunction, jsEventCode)

	s = jsEventTagBegin + jsEventCode

	return s

def extScriptUrlReplaceFunction(match):
	#extScriptPatBegin = match.group("extScriptPatBegin")
	#extScriptUrl = match.group("extScriptUrl")
	#absoluteUrl = zkmUrl.constructAbsoluteUrl(urlBase, extScriptUrl)
	#s = extScriptPatBegin + absoluteUrl

	s = ""	#remove external scripts for now
	return s

def proxyToCacheReplaceFunction(match):
	return r'/zkm/bin/zkmCache'

def directUserToCache(parmsDict, s):
	#convert all URLs in this string with /zkm/bin/zkmProxy in them to URLs with
	#/zkm/bin/zkmCache instead, so that the user will directed to our fetch from cache
	#function versus the proxy function. The way you get to this code is by using the search
	#functionality.
	global urlBase
	global httpServerPort
	global httpServerName
	global graphicsCaching
	global binariesFolder

	urlBase = parmsDict["zkmBase"]
	httpServerPort = parmsDict["serverPort"]
	httpServerName = parmsDict["serverName"]
	graphicsCaching = parmsDict["graphicsCaching"]
	binariesFolder = parmsDict["binariesFolder"]

	s = proxySubstitutions(s)

	s = proxyUrlPat.sub(proxyToCacheReplaceFunction, s)

	return s

def proxySubstitutions(s):
	#fix anchor urls
	s = anchorPat.sub(anchorReplaceFunction, s)

	#fix meta refresh url
	s = metaRefreshPat.sub(metaRefreshReplaceFunction, s)

	#find external script tags--do this before the other tags with "src" in them
	s = extScriptPat.sub(extScriptUrlReplaceFunction, s)

	#fix src tag urls--will potentially have to do something different when we start keeping script tags
	#because these have "src" attributes as well.
	s = srcPat.sub(srcReplaceFunction, s)

	#fix image objects' (dot) src properties (Javascript).
	#NOTE: shares replace function (srcReplaceFunction) with src tag urls
	s = imageObjectSrcPropertyPat.sub(srcReplaceFunction, s)

	#fix events (Javascript)--all events starting with "on"--see global list
	s = javascriptEventPat.sub(jsEventReplaceFunction, s)

	#fix body tag background url
	s = bodyBgrndPat.sub(bgrndUrlReplaceFunction, s)

	#fix form action urls
	s = formActionPat.sub(formActionUrlReplaceFunction, s)

	return s

def proxifyHtml(parmsDict, s):
	global urlBase
	global httpServerPort
	global httpServerName
	global graphicsCaching
	global binariesFolder

	urlBase = parmsDict["zkmBase"]
	httpServerPort = parmsDict["serverPort"]
	httpServerName = parmsDict["serverName"]
	graphicsCaching = parmsDict["graphicsCaching"]
	binariesFolder = parmsDict["binariesFolder"]

	s = proxySubstitutions(s)

	return s

def formatError(s):
	return '<CENTER><FONT SIZE="4" COLOR="RED">' + s + '</FONT></CENTER>'

def formatMessage(s):
	return '<CENTER><FONT SIZE="4">' + s + '</FONT></CENTER>'

